<?php
/**
 * Plugin Name: ZimmWriter OG Image Manager
 * Plugin URI: https://yourwebsite.com/plugins/zimmwriter-og-image-manager
 * Description: Manages Open Graph image properties for WordPress posts through the REST API
 * Version: 1.0.0
 * Author: Your Name
 * Author URI: https://yourwebsite.com
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain: zimmwriter-og-image
 * Domain Path: /languages
 *
 * DISCLAIMER:
 * THIS PLUGIN IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE PLUGIN OR THE USE OR OTHER DEALINGS IN THE
 * PLUGIN.
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

/**
 * Define plugin constants
 */
define('ZIMMWRITER_OG_IMAGE_VERSION', '1.0.0');
define('ZIMMWRITER_OG_IMAGE_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('ZIMMWRITER_OG_IMAGE_PLUGIN_URL', plugin_dir_url(__FILE__));

/**
 * The core plugin class
 */
class ZimmWriter_OG_Image_Manager {

    /**
     * Initialize the plugin
     */
    public function __construct() {
        // Register hooks
        add_action('init', array($this, 'register_og_image_meta_fields'));
        
        // Optional: Add admin UI hooks if needed
        add_action('add_meta_boxes', array($this, 'add_og_image_meta_box'));
        add_action('save_post', array($this, 'save_og_image_meta'));
        
        // Use buffer method ONLY - do not use wp_head hook to avoid duplication
        add_action('template_redirect', array($this, 'buffer_html_output'), 1);
    }

    /**
     * Start output buffering to modify HTML output
     */
    public function buffer_html_output() {
        if (is_singular('post')) {
            ob_start(array($this, 'process_html_buffer'));
        }
    }

    /**
     * Process the HTML buffer to insert OG tags at the very top
     */
    public function process_html_buffer($buffer) {
        if (!is_singular('post')) {
            return $buffer;
        }

        $post_id = get_the_ID();
        $og_image_enabled = get_post_meta($post_id, 'og_image_enabled', true);
        
        // Only proceed if feature is enabled for this post
        if (empty($og_image_enabled)) {
            return $buffer;
        }

        $og_image = get_post_meta($post_id, 'og_image', true);
        if (empty($og_image)) {
            return $buffer;
        }

        // Prepare OG image tags
        $og_tags = '';
        $og_tags .= '<meta property="og:image" content="' . esc_url($og_image) . '" />' . "\n";
        $og_tags .= '<meta property="og:image:secure_url" content="' . esc_url($og_image) . '" />' . "\n";
        
        $width = get_post_meta($post_id, 'og_image_width', true);
        if (!empty($width)) {
            $og_tags .= '<meta property="og:image:width" content="' . esc_attr($width) . '" />' . "\n";
        }
        
        $height = get_post_meta($post_id, 'og_image_height', true);
        if (!empty($height)) {
            $og_tags .= '<meta property="og:image:height" content="' . esc_attr($height) . '" />' . "\n";
        }

        // Insert OG tags right after the <head> tag
        $buffer = preg_replace('/<head\b[^>]*>/', '$0' . "\n" . $og_tags, $buffer, 1);
        
        return $buffer;
    }

    /**
     * Register og_image meta fields with WordPress REST API
     */
    public function register_og_image_meta_fields() {
        // Register meta fields for all og:image related properties
        $og_image_fields = array(
            'og_image' => array(
                'type' => 'string',
                'description' => 'The Open Graph image URL'
            ),
            'og_image_width' => array(
                'type' => 'integer',
                'description' => 'The Open Graph image width'
            ),
            'og_image_height' => array(
                'type' => 'integer',
                'description' => 'The Open Graph image height'
            ),
            'og_image_enabled' => array(
                'type' => 'boolean',
                'description' => 'Whether custom OG image is enabled for this post'
            )
        );
        
        foreach ($og_image_fields as $field => $config) {
            register_post_meta('post', $field, array(
                'show_in_rest' => true,
                'single' => true,
                'type' => $config['type'],
                'description' => $config['description'],
            ));
            
            // Register REST field for each meta
            register_rest_field('post', $field, array(
                'get_callback' => function($post_arr) use ($field) {
                    return get_post_meta($post_arr['id'], $field, true);
                },
                'update_callback' => function($value, $post) use ($field, $config) {
                    if (!empty($value)) {
                        if ($config['type'] === 'string') {
                            update_post_meta($post->ID, $field, esc_url_raw($value));
                        } else {
                            update_post_meta($post->ID, $field, absint($value));
                        }
                    } else {
                        delete_post_meta($post->ID, $field);
                    }
                },
                'schema' => array(
                    'description' => $config['description'],
                    'type' => $config['type'],
                    $config['type'] === 'string' ? 'format' : null => $config['type'] === 'string' ? 'uri' : null,
                ),
            ));
        }
    }

    /**
     * Add a meta box to the post edit screen
     */
    public function add_og_image_meta_box() {
        add_meta_box(
            'zimmwriter_og_image_meta_box',
            'Open Graph Image',
            array($this, 'render_og_image_meta_box'),
            'post',
            'side',
            'default'
        );
    }

    /**
     * Render the meta box content
     */
    public function render_og_image_meta_box($post) {
        // Add nonce for security
        wp_nonce_field('zimmwriter_og_image_meta_box', 'zimmwriter_og_image_meta_box_nonce');

        // Get current values
        $og_image = get_post_meta($post->ID, 'og_image', true);
        $og_image_enabled = !empty(get_post_meta($post->ID, 'og_image_enabled', true));
        
        // Output fields
        ?>
        <p>
            <label>
                <input type="checkbox" id="og_image_enabled" name="og_image_enabled" value="1" <?php checked($og_image_enabled, true); ?>>
                <?php _e('Enable custom OG Image', 'zimmwriter-og-image'); ?>
            </label>
        </p>
        
        <?php if (!empty($og_image)) : ?>
            <p>
                <strong><?php _e('Current OG Image:', 'zimmwriter-og-image'); ?></strong>
                <span class="description"><?php echo esc_url($og_image); ?></span>
            </p>
            <div class="og-image-preview" style="margin-top: 10px;">
                <img src="<?php echo esc_url($og_image); ?>" style="max-width: 100%; height: auto;">
            </div>
        <?php else : ?>
            <p><em><?php _e('No custom OG image set. Submit via REST API.', 'zimmwriter-og-image'); ?></em></p>
        <?php endif; ?>
        
        <p class="description">
            <?php _e('Custom OG image properties can only be set via REST API. This toggle only enables/disables the feature for this post.', 'zimmwriter-og-image'); ?>
        </p>
        <?php
    }

    /**
     * Save the meta box data
     */
    public function save_og_image_meta($post_id) {
        // Check if our nonce is set and verify it
        if (!isset($_POST['zimmwriter_og_image_meta_box_nonce']) || 
            !wp_verify_nonce($_POST['zimmwriter_og_image_meta_box_nonce'], 'zimmwriter_og_image_meta_box')) {
            return;
        }
        
        // Check the user's permissions
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        
        // Check the user's permissions
        if ('post' === $_POST['post_type']) {
            if (!current_user_can('edit_post', $post_id)) {
                return;
            }
        }
        
        // Update only the enabled/disabled status
        $og_image_enabled = isset($_POST['og_image_enabled']) ? '1' : '';
        update_post_meta($post_id, 'og_image_enabled', $og_image_enabled);
    }
}

// Initialize the plugin
$zimmwriter_og_image_manager = new ZimmWriter_OG_Image_Manager();